# these will create pass through shunts (on the responder?)
../../guestbin/ping-once.sh --forget -I 192.1.3.33 192.1.2.23
../../guestbin/ping-once.sh --up -I 192.1.3.33 192.1.2.23
../../guestbin/ping-once.sh --up -I 192.1.3.33 192.1.2.45
echo run done
