/testing/guestbin/swan-prep
ip addr add 192.1.3.1/24 dev eth0
ip addr add 192.1.3.2/24 dev eth0
ip addr add 192.1.3.3/24 dev eth0
ip addr add 192.1.3.4/24 dev eth0
ip addr add 192.1.3.5/24 dev eth0
ip addr add 192.1.3.6/24 dev eth0
ip addr add 192.1.3.7/24 dev eth0
ip addr add 192.1.3.8/24 dev eth0
ip addr add 192.1.3.9/24 dev eth0
ip addr add 192.1.3.10/24 dev eth0
ip addr add 192.1.3.11/24 dev eth0
ip addr add 192.1.3.12/24 dev eth0
ip addr add 192.1.3.13/24 dev eth0
ip addr add 192.1.3.14/24 dev eth0
ip addr add 192.1.3.15/24 dev eth0
ip addr add 192.1.3.16/24 dev eth0
ip addr add 192.1.3.17/24 dev eth0
ip addr add 192.1.3.18/24 dev eth0
ip addr add 192.1.3.19/24 dev eth0
ip addr add 192.1.3.20/24 dev eth0
ip addr add 192.1.3.21/24 dev eth0
ip addr add 192.1.3.22/24 dev eth0
ip addr add 192.1.3.23/24 dev eth0
ip addr add 192.1.3.24/24 dev eth0
ip addr add 192.1.3.25/24 dev eth0
ip addr add 192.1.3.26/24 dev eth0
ip addr add 192.1.3.27/24 dev eth0
ip addr add 192.1.3.28/24 dev eth0
ip addr add 192.1.3.29/24 dev eth0
ip addr add 192.1.3.30/24 dev eth0
ip addr add 192.1.3.31/24 dev eth0
ip addr add 192.1.3.32/24 dev eth0
ip addr add 192.1.3.33/24 dev eth0
ip addr add 192.1.3.34/24 dev eth0
ip addr add 192.1.3.35/24 dev eth0
ip addr add 192.1.3.36/24 dev eth0
ip addr add 192.1.3.37/24 dev eth0
ip addr add 192.1.3.38/24 dev eth0
ip addr add 192.1.3.39/24 dev eth0
ip addr add 192.1.3.40/24 dev eth0
ip addr add 192.1.3.41/24 dev eth0
ip addr add 192.1.3.42/24 dev eth0
ip addr add 192.1.3.43/24 dev eth0
ip addr add 192.1.3.44/24 dev eth0
ip addr add 192.1.3.45/24 dev eth0
ip addr add 192.1.3.46/24 dev eth0
ip addr add 192.1.3.47/24 dev eth0
ip addr add 192.1.3.48/24 dev eth0
ip addr add 192.1.3.49/24 dev eth0
ip addr add 192.1.3.50/24 dev eth0
ip addr add 192.1.3.51/24 dev eth0
ip addr add 192.1.3.52/24 dev eth0
ip addr add 192.1.3.53/24 dev eth0
ip addr add 192.1.3.54/24 dev eth0
ip addr add 192.1.3.55/24 dev eth0
ip addr add 192.1.3.56/24 dev eth0
ip addr add 192.1.3.57/24 dev eth0
ip addr add 192.1.3.58/24 dev eth0
ip addr add 192.1.3.59/24 dev eth0
ip addr add 192.1.3.60/24 dev eth0
ip addr add 192.1.3.61/24 dev eth0
ip addr add 192.1.3.62/24 dev eth0
ip addr add 192.1.3.63/24 dev eth0
ip addr add 192.1.3.64/24 dev eth0
ip addr add 192.1.3.65/24 dev eth0
ip addr add 192.1.3.66/24 dev eth0
ip addr add 192.1.3.67/24 dev eth0
ip addr add 192.1.3.68/24 dev eth0
ip addr add 192.1.3.69/24 dev eth0
ip addr add 192.1.3.70/24 dev eth0
ip addr add 192.1.3.71/24 dev eth0
ip addr add 192.1.3.72/24 dev eth0
ip addr add 192.1.3.73/24 dev eth0
ip addr add 192.1.3.74/24 dev eth0
ip addr add 192.1.3.75/24 dev eth0
ip addr add 192.1.3.76/24 dev eth0
ip addr add 192.1.3.77/24 dev eth0
ip addr add 192.1.3.78/24 dev eth0
ip addr add 192.1.3.79/24 dev eth0
ip addr add 192.1.3.80/24 dev eth0
ip addr add 192.1.3.81/24 dev eth0
ip addr add 192.1.3.82/24 dev eth0
ip addr add 192.1.3.83/24 dev eth0
ip addr add 192.1.3.84/24 dev eth0
ip addr add 192.1.3.85/24 dev eth0
ip addr add 192.1.3.86/24 dev eth0
ip addr add 192.1.3.87/24 dev eth0
ip addr add 192.1.3.88/24 dev eth0
ip addr add 192.1.3.89/24 dev eth0
ip addr add 192.1.3.90/24 dev eth0
ip addr add 192.1.3.91/24 dev eth0
ip addr add 192.1.3.92/24 dev eth0
ip addr add 192.1.3.93/24 dev eth0
ip addr add 192.1.3.94/24 dev eth0
ip addr add 192.1.3.95/24 dev eth0
ip addr add 192.1.3.96/24 dev eth0
ip addr add 192.1.3.97/24 dev eth0
ip addr add 192.1.3.98/24 dev eth0
ip addr add 192.1.3.99/24 dev eth0
ip addr add 192.1.3.100/24 dev eth0
ip addr add 192.1.3.101/24 dev eth0
ip addr add 192.1.3.102/24 dev eth0
ip addr add 192.1.3.103/24 dev eth0
ip addr add 192.1.3.104/24 dev eth0
ip addr add 192.1.3.105/24 dev eth0
ip addr add 192.1.3.106/24 dev eth0
ip addr add 192.1.3.107/24 dev eth0
ip addr add 192.1.3.108/24 dev eth0
ip addr add 192.1.3.109/24 dev eth0
ip addr add 192.1.3.110/24 dev eth0
ip addr add 192.1.3.111/24 dev eth0
ip addr add 192.1.3.112/24 dev eth0
ip addr add 192.1.3.113/24 dev eth0
ip addr add 192.1.3.114/24 dev eth0
ip addr add 192.1.3.115/24 dev eth0
ip addr add 192.1.3.116/24 dev eth0
ip addr add 192.1.3.117/24 dev eth0
ip addr add 192.1.3.118/24 dev eth0
ip addr add 192.1.3.119/24 dev eth0
ip addr add 192.1.3.120/24 dev eth0
ip addr add 192.1.3.121/24 dev eth0
ip addr add 192.1.3.122/24 dev eth0
ip addr add 192.1.3.123/24 dev eth0
ip addr add 192.1.3.124/24 dev eth0
ip addr add 192.1.3.125/24 dev eth0
ip addr add 192.1.3.126/24 dev eth0
ip addr add 192.1.3.127/24 dev eth0
ip addr add 192.1.3.128/24 dev eth0
ip addr add 192.1.3.129/24 dev eth0
ip addr add 192.1.3.130/24 dev eth0
ip addr add 192.1.3.131/24 dev eth0
ip addr add 192.1.3.132/24 dev eth0
ip addr add 192.1.3.133/24 dev eth0
ip addr add 192.1.3.134/24 dev eth0
ip addr add 192.1.3.135/24 dev eth0
ip addr add 192.1.3.136/24 dev eth0
ip addr add 192.1.3.137/24 dev eth0
ip addr add 192.1.3.138/24 dev eth0
ip addr add 192.1.3.139/24 dev eth0
ip addr add 192.1.3.140/24 dev eth0
ip addr add 192.1.3.141/24 dev eth0
ip addr add 192.1.3.142/24 dev eth0
ip addr add 192.1.3.143/24 dev eth0
ip addr add 192.1.3.144/24 dev eth0
ip addr add 192.1.3.145/24 dev eth0
ip addr add 192.1.3.146/24 dev eth0
ip addr add 192.1.3.147/24 dev eth0
ip addr add 192.1.3.148/24 dev eth0
ip addr add 192.1.3.149/24 dev eth0
ip addr add 192.1.3.150/24 dev eth0
ip addr add 192.1.3.151/24 dev eth0
ip addr add 192.1.3.152/24 dev eth0
ip addr add 192.1.3.153/24 dev eth0
ip addr add 192.1.3.154/24 dev eth0
ip addr add 192.1.3.155/24 dev eth0
ip addr add 192.1.3.156/24 dev eth0
ip addr add 192.1.3.157/24 dev eth0
ip addr add 192.1.3.158/24 dev eth0
ip addr add 192.1.3.159/24 dev eth0
ip addr add 192.1.3.160/24 dev eth0
ip addr add 192.1.3.161/24 dev eth0
ip addr add 192.1.3.162/24 dev eth0
ip addr add 192.1.3.163/24 dev eth0
ip addr add 192.1.3.164/24 dev eth0
ip addr add 192.1.3.165/24 dev eth0
ip addr add 192.1.3.166/24 dev eth0
ip addr add 192.1.3.167/24 dev eth0
ip addr add 192.1.3.168/24 dev eth0
ip addr add 192.1.3.169/24 dev eth0
ip addr add 192.1.3.170/24 dev eth0
ip addr add 192.1.3.171/24 dev eth0
ip addr add 192.1.3.172/24 dev eth0
ip addr add 192.1.3.173/24 dev eth0
ip addr add 192.1.3.174/24 dev eth0
ip addr add 192.1.3.175/24 dev eth0
ip addr add 192.1.3.176/24 dev eth0
ip addr add 192.1.3.177/24 dev eth0
ip addr add 192.1.3.178/24 dev eth0
ip addr add 192.1.3.179/24 dev eth0
ip addr add 192.1.3.180/24 dev eth0
ip addr add 192.1.3.181/24 dev eth0
ip addr add 192.1.3.182/24 dev eth0
ip addr add 192.1.3.183/24 dev eth0
ip addr add 192.1.3.184/24 dev eth0
ip addr add 192.1.3.185/24 dev eth0
ip addr add 192.1.3.186/24 dev eth0
ip addr add 192.1.3.187/24 dev eth0
ip addr add 192.1.3.188/24 dev eth0
ip addr add 192.1.3.189/24 dev eth0
ip addr add 192.1.3.190/24 dev eth0
ip addr add 192.1.3.191/24 dev eth0
ip addr add 192.1.3.192/24 dev eth0
ip addr add 192.1.3.193/24 dev eth0
ip addr add 192.1.3.194/24 dev eth0
ip addr add 192.1.3.195/24 dev eth0
ip addr add 192.1.3.196/24 dev eth0
ip addr add 192.1.3.197/24 dev eth0
ip addr add 192.1.3.198/24 dev eth0
ip addr add 192.1.3.199/24 dev eth0
ip addr add 192.1.3.200/24 dev eth0
ip addr add 192.1.3.201/24 dev eth0
ip addr add 192.1.3.202/24 dev eth0
ip addr add 192.1.3.203/24 dev eth0
ip addr add 192.1.3.204/24 dev eth0
ip addr add 192.1.3.205/24 dev eth0
ip addr add 192.1.3.206/24 dev eth0
ip addr add 192.1.3.207/24 dev eth0
ip addr add 192.1.3.208/24 dev eth0
ip addr add 192.1.3.209/24 dev eth0
ip addr add 192.1.3.210/24 dev eth0
ip addr add 192.1.3.211/24 dev eth0
ip addr add 192.1.3.212/24 dev eth0
ip addr add 192.1.3.213/24 dev eth0
ip addr add 192.1.3.214/24 dev eth0
ip addr add 192.1.3.215/24 dev eth0
ip addr add 192.1.3.216/24 dev eth0
ip addr add 192.1.3.217/24 dev eth0
ip addr add 192.1.3.218/24 dev eth0
ip addr add 192.1.3.219/24 dev eth0
ip addr add 192.1.3.220/24 dev eth0
ip addr add 192.1.3.221/24 dev eth0
ip addr add 192.1.3.222/24 dev eth0
ip addr add 192.1.3.223/24 dev eth0
ip addr add 192.1.3.224/24 dev eth0
ip addr add 192.1.3.225/24 dev eth0
ip addr add 192.1.3.226/24 dev eth0
ip addr add 192.1.3.227/24 dev eth0
ip addr add 192.1.3.228/24 dev eth0
ip addr add 192.1.3.229/24 dev eth0
ip addr add 192.1.3.230/24 dev eth0
ip addr add 192.1.3.231/24 dev eth0
ip addr add 192.1.3.232/24 dev eth0
ip addr add 192.1.3.233/24 dev eth0
ip addr add 192.1.3.234/24 dev eth0
ip addr add 192.1.3.235/24 dev eth0
ip addr add 192.1.3.236/24 dev eth0
ip addr add 192.1.3.237/24 dev eth0
ip addr add 192.1.3.238/24 dev eth0
ip addr add 192.1.3.239/24 dev eth0
ip addr add 192.1.3.240/24 dev eth0
ip addr add 192.1.3.241/24 dev eth0
ip addr add 192.1.3.242/24 dev eth0
ip addr add 192.1.3.243/24 dev eth0
ip addr add 192.1.3.244/24 dev eth0
ip addr add 192.1.3.245/24 dev eth0
ip addr add 192.1.3.246/24 dev eth0
ip addr add 192.1.3.247/24 dev eth0
ip addr add 192.1.3.248/24 dev eth0
ip addr add 192.1.3.249/24 dev eth0
ip addr add 192.1.3.250/24 dev eth0
ip addr add 192.1.3.251/24 dev eth0
ip addr add 192.1.3.252/24 dev eth0
ipsec start
../../guestbin/wait-until-pluto-started
ipsec auto --add road-eastnet-nonat-1
ipsec auto --add road-eastnet-nonat-2
ipsec auto --add road-eastnet-nonat-3
ipsec auto --add road-eastnet-nonat-4
ipsec auto --add road-eastnet-nonat-5
ipsec auto --add road-eastnet-nonat-6
ipsec auto --add road-eastnet-nonat-7
ipsec auto --add road-eastnet-nonat-8
ipsec auto --add road-eastnet-nonat-9
ipsec auto --add road-eastnet-nonat-10
ipsec auto --add road-eastnet-nonat-11
ipsec auto --add road-eastnet-nonat-12
ipsec auto --add road-eastnet-nonat-13
ipsec auto --add road-eastnet-nonat-14
ipsec auto --add road-eastnet-nonat-15
ipsec auto --add road-eastnet-nonat-16
ipsec auto --add road-eastnet-nonat-17
ipsec auto --add road-eastnet-nonat-18
ipsec auto --add road-eastnet-nonat-19
ipsec auto --add road-eastnet-nonat-20
ipsec auto --add road-eastnet-nonat-21
ipsec auto --add road-eastnet-nonat-22
ipsec auto --add road-eastnet-nonat-23
ipsec auto --add road-eastnet-nonat-24
ipsec auto --add road-eastnet-nonat-25
ipsec auto --add road-eastnet-nonat-26
ipsec auto --add road-eastnet-nonat-27
ipsec auto --add road-eastnet-nonat-28
ipsec auto --add road-eastnet-nonat-29
ipsec auto --add road-eastnet-nonat-30
ipsec auto --add road-eastnet-nonat-31
ipsec auto --add road-eastnet-nonat-32
ipsec auto --add road-eastnet-nonat-33
ipsec auto --add road-eastnet-nonat-34
ipsec auto --add road-eastnet-nonat-35
ipsec auto --add road-eastnet-nonat-36
ipsec auto --add road-eastnet-nonat-37
ipsec auto --add road-eastnet-nonat-38
ipsec auto --add road-eastnet-nonat-39
ipsec auto --add road-eastnet-nonat-40
ipsec auto --add road-eastnet-nonat-41
ipsec auto --add road-eastnet-nonat-42
ipsec auto --add road-eastnet-nonat-43
ipsec auto --add road-eastnet-nonat-44
ipsec auto --add road-eastnet-nonat-45
ipsec auto --add road-eastnet-nonat-46
ipsec auto --add road-eastnet-nonat-47
ipsec auto --add road-eastnet-nonat-48
ipsec auto --add road-eastnet-nonat-49
ipsec auto --add road-eastnet-nonat-50
ipsec auto --add road-eastnet-nonat-51
ipsec auto --add road-eastnet-nonat-52
ipsec auto --add road-eastnet-nonat-53
ipsec auto --add road-eastnet-nonat-54
ipsec auto --add road-eastnet-nonat-55
ipsec auto --add road-eastnet-nonat-56
ipsec auto --add road-eastnet-nonat-57
ipsec auto --add road-eastnet-nonat-58
ipsec auto --add road-eastnet-nonat-59
ipsec auto --add road-eastnet-nonat-60
ipsec auto --add road-eastnet-nonat-61
ipsec auto --add road-eastnet-nonat-62
ipsec auto --add road-eastnet-nonat-63
ipsec auto --add road-eastnet-nonat-64
ipsec auto --add road-eastnet-nonat-65
ipsec auto --add road-eastnet-nonat-66
ipsec auto --add road-eastnet-nonat-67
ipsec auto --add road-eastnet-nonat-68
ipsec auto --add road-eastnet-nonat-69
ipsec auto --add road-eastnet-nonat-70
ipsec auto --add road-eastnet-nonat-71
ipsec auto --add road-eastnet-nonat-72
ipsec auto --add road-eastnet-nonat-73
ipsec auto --add road-eastnet-nonat-74
ipsec auto --add road-eastnet-nonat-75
ipsec auto --add road-eastnet-nonat-76
ipsec auto --add road-eastnet-nonat-77
ipsec auto --add road-eastnet-nonat-78
ipsec auto --add road-eastnet-nonat-79
ipsec auto --add road-eastnet-nonat-80
ipsec auto --add road-eastnet-nonat-81
ipsec auto --add road-eastnet-nonat-82
ipsec auto --add road-eastnet-nonat-83
ipsec auto --add road-eastnet-nonat-84
ipsec auto --add road-eastnet-nonat-85
ipsec auto --add road-eastnet-nonat-86
ipsec auto --add road-eastnet-nonat-87
ipsec auto --add road-eastnet-nonat-88
ipsec auto --add road-eastnet-nonat-89
ipsec auto --add road-eastnet-nonat-90
ipsec auto --add road-eastnet-nonat-91
ipsec auto --add road-eastnet-nonat-92
ipsec auto --add road-eastnet-nonat-93
ipsec auto --add road-eastnet-nonat-94
ipsec auto --add road-eastnet-nonat-95
ipsec auto --add road-eastnet-nonat-96
ipsec auto --add road-eastnet-nonat-97
ipsec auto --add road-eastnet-nonat-98
ipsec auto --add road-eastnet-nonat-99
ipsec auto --add road-eastnet-nonat-100
ipsec auto --add road-eastnet-nonat-101
ipsec auto --add road-eastnet-nonat-102
ipsec auto --add road-eastnet-nonat-103
ipsec auto --add road-eastnet-nonat-104
ipsec auto --add road-eastnet-nonat-105
ipsec auto --add road-eastnet-nonat-106
ipsec auto --add road-eastnet-nonat-107
ipsec auto --add road-eastnet-nonat-108
ipsec auto --add road-eastnet-nonat-109
ipsec auto --add road-eastnet-nonat-110
ipsec auto --add road-eastnet-nonat-111
ipsec auto --add road-eastnet-nonat-112
ipsec auto --add road-eastnet-nonat-113
ipsec auto --add road-eastnet-nonat-114
ipsec auto --add road-eastnet-nonat-115
ipsec auto --add road-eastnet-nonat-116
ipsec auto --add road-eastnet-nonat-117
ipsec auto --add road-eastnet-nonat-118
ipsec auto --add road-eastnet-nonat-119
ipsec auto --add road-eastnet-nonat-120
ipsec auto --add road-eastnet-nonat-121
ipsec auto --add road-eastnet-nonat-122
ipsec auto --add road-eastnet-nonat-123
ipsec auto --add road-eastnet-nonat-124
ipsec auto --add road-eastnet-nonat-125
ipsec auto --add road-eastnet-nonat-126
ipsec auto --add road-eastnet-nonat-127
ipsec auto --add road-eastnet-nonat-128
ipsec auto --add road-eastnet-nonat-129
ipsec auto --add road-eastnet-nonat-130
ipsec auto --add road-eastnet-nonat-131
ipsec auto --add road-eastnet-nonat-132
ipsec auto --add road-eastnet-nonat-133
ipsec auto --add road-eastnet-nonat-134
ipsec auto --add road-eastnet-nonat-135
ipsec auto --add road-eastnet-nonat-136
ipsec auto --add road-eastnet-nonat-137
ipsec auto --add road-eastnet-nonat-138
ipsec auto --add road-eastnet-nonat-139
ipsec auto --add road-eastnet-nonat-140
ipsec auto --add road-eastnet-nonat-141
ipsec auto --add road-eastnet-nonat-142
ipsec auto --add road-eastnet-nonat-143
ipsec auto --add road-eastnet-nonat-144
ipsec auto --add road-eastnet-nonat-145
ipsec auto --add road-eastnet-nonat-146
ipsec auto --add road-eastnet-nonat-147
ipsec auto --add road-eastnet-nonat-148
ipsec auto --add road-eastnet-nonat-149
ipsec auto --add road-eastnet-nonat-150
ipsec auto --add road-eastnet-nonat-151
ipsec auto --add road-eastnet-nonat-152
ipsec auto --add road-eastnet-nonat-153
ipsec auto --add road-eastnet-nonat-154
ipsec auto --add road-eastnet-nonat-155
ipsec auto --add road-eastnet-nonat-156
ipsec auto --add road-eastnet-nonat-157
ipsec auto --add road-eastnet-nonat-158
ipsec auto --add road-eastnet-nonat-159
ipsec auto --add road-eastnet-nonat-160
ipsec auto --add road-eastnet-nonat-161
ipsec auto --add road-eastnet-nonat-162
ipsec auto --add road-eastnet-nonat-163
ipsec auto --add road-eastnet-nonat-164
ipsec auto --add road-eastnet-nonat-165
ipsec auto --add road-eastnet-nonat-166
ipsec auto --add road-eastnet-nonat-167
ipsec auto --add road-eastnet-nonat-168
ipsec auto --add road-eastnet-nonat-169
ipsec auto --add road-eastnet-nonat-170
ipsec auto --add road-eastnet-nonat-171
ipsec auto --add road-eastnet-nonat-172
ipsec auto --add road-eastnet-nonat-173
ipsec auto --add road-eastnet-nonat-174
ipsec auto --add road-eastnet-nonat-175
ipsec auto --add road-eastnet-nonat-176
ipsec auto --add road-eastnet-nonat-177
ipsec auto --add road-eastnet-nonat-178
ipsec auto --add road-eastnet-nonat-179
ipsec auto --add road-eastnet-nonat-180
ipsec auto --add road-eastnet-nonat-181
ipsec auto --add road-eastnet-nonat-182
ipsec auto --add road-eastnet-nonat-183
ipsec auto --add road-eastnet-nonat-184
ipsec auto --add road-eastnet-nonat-185
ipsec auto --add road-eastnet-nonat-186
ipsec auto --add road-eastnet-nonat-187
ipsec auto --add road-eastnet-nonat-188
ipsec auto --add road-eastnet-nonat-189
ipsec auto --add road-eastnet-nonat-190
ipsec auto --add road-eastnet-nonat-191
ipsec auto --add road-eastnet-nonat-192
ipsec auto --add road-eastnet-nonat-193
ipsec auto --add road-eastnet-nonat-194
ipsec auto --add road-eastnet-nonat-195
ipsec auto --add road-eastnet-nonat-196
ipsec auto --add road-eastnet-nonat-197
ipsec auto --add road-eastnet-nonat-198
ipsec auto --add road-eastnet-nonat-199
ipsec auto --add road-eastnet-nonat-200
ipsec auto --add road-eastnet-nonat-201
ipsec auto --add road-eastnet-nonat-202
ipsec auto --add road-eastnet-nonat-203
ipsec auto --add road-eastnet-nonat-204
ipsec auto --add road-eastnet-nonat-205
ipsec auto --add road-eastnet-nonat-206
ipsec auto --add road-eastnet-nonat-207
ipsec auto --add road-eastnet-nonat-208
ipsec auto --add road-eastnet-nonat-209
ipsec auto --add road-eastnet-nonat-210
ipsec auto --add road-eastnet-nonat-211
ipsec auto --add road-eastnet-nonat-212
ipsec auto --add road-eastnet-nonat-213
ipsec auto --add road-eastnet-nonat-214
ipsec auto --add road-eastnet-nonat-215
ipsec auto --add road-eastnet-nonat-216
ipsec auto --add road-eastnet-nonat-217
ipsec auto --add road-eastnet-nonat-218
ipsec auto --add road-eastnet-nonat-219
ipsec auto --add road-eastnet-nonat-220
ipsec auto --add road-eastnet-nonat-221
ipsec auto --add road-eastnet-nonat-222
ipsec auto --add road-eastnet-nonat-223
ipsec auto --add road-eastnet-nonat-224
ipsec auto --add road-eastnet-nonat-225
ipsec auto --add road-eastnet-nonat-226
ipsec auto --add road-eastnet-nonat-227
ipsec auto --add road-eastnet-nonat-228
ipsec auto --add road-eastnet-nonat-229
ipsec auto --add road-eastnet-nonat-230
ipsec auto --add road-eastnet-nonat-231
ipsec auto --add road-eastnet-nonat-232
ipsec auto --add road-eastnet-nonat-233
ipsec auto --add road-eastnet-nonat-234
ipsec auto --add road-eastnet-nonat-235
ipsec auto --add road-eastnet-nonat-236
ipsec auto --add road-eastnet-nonat-237
ipsec auto --add road-eastnet-nonat-238
ipsec auto --add road-eastnet-nonat-239
ipsec auto --add road-eastnet-nonat-240
ipsec auto --add road-eastnet-nonat-241
ipsec auto --add road-eastnet-nonat-242
ipsec auto --add road-eastnet-nonat-243
ipsec auto --add road-eastnet-nonat-244
ipsec auto --add road-eastnet-nonat-245
ipsec auto --add road-eastnet-nonat-246
ipsec auto --add road-eastnet-nonat-247
ipsec auto --add road-eastnet-nonat-248
ipsec auto --add road-eastnet-nonat-249
ipsec auto --add road-eastnet-nonat-250
ipsec auto --add road-eastnet-nonat-251
ipsec auto --add road-eastnet-nonat-252
echo "initdone"
